@@ -32,7 +32,9 @@ CLASSES AND METHODS
 """
 
 import numpy as np
+import matplotlib.pyplot as plt
 import scipy.stats as stats
+from scipy.spatial.distance import mahalanobis
 
 class ClusterData:
 	"""
@@ -158,6 +160,78 @@ class ClusterData:
 			raise Exception('No data has been generated. Use ClusterData.generate_data to generate data.')
 		else:
 			np.savetxt(filename, self.data, delimiter=',')
+
+
+	def modify_cov_structure(self,cov,cov_inv, cluster_axes, cluster_sd):
+		"""
+		Modify the covariance structure of a ClusterData object.
+		
+		Need to specify ALL attributes of the covariance structure.
+		"""
+		self.cov = cov
+		self.cov_inv = cov_inv
+		self.cluster_axes = cluster_axes
+		self.cluster_sd = cluster_sd
+
+
+	def draw_model(self,s=30,alpha_max=0.125, sd_max_bare=2,sd_max_density=4, mode='bare', h=0.1,dr=0.1,
+				   cluster_labels=True):
+		"""
+		Plot a sketch of the geometry underlying a probabilistic mixture model sampled by a ClusterData object.
+		
+		Parameters
+		----------
+		clusterdata : ClusterData object
+		s : float
+		alpha_max :
+		sd_max_bare :
+		sd_max_density : 
+		mode : str
+			Specify the desired type of drawing. The options are
+			   'bare' : draw each cluster as a central point and elliptical contour,
+						which is located two standard deviations away from the mean
+			   'density' : draw each cluster as a probability density that fades out
+						   gradually until the elliptical contour four standard
+						   deviations away from the mean
+			   'arrows' : as in 'bare', except that arrows are added along the principal axes
+						NOT IMPLEMENTED YET
+		"""
+
+		centers = self.centers
+		sd = self.cluster_sd
+		axes = self.cluster_axes
+
+		for i in range(centers.shape[0]):
+			# plot cluster center
+			x_c = centers[i,0]
+			y_c = centers[i,1]
+			plt.scatter(x_c,y_c,c='k',s=s)
+			if cluster_labels:
+				plt.text(x_c,y_c,' ' + str(i),fontsize=16,va='center')
+			
+			if mode=='bare':
+				# plot only a single elliptical contour and the cluster center
+				
+				r = sd_max_bare
+				x = [r*sd[i][0]*np.cos(theta) for theta in np.arange(0,2*np.pi+h,h)]
+				y = [r*sd[i][1]*np.sin(theta) for theta in np.arange(0,2*np.pi+h,h)]
+				Z = np.transpose(axes[i]) @ np.array([x,y])
+				
+				plt.plot(Z[0]+x_c,Z[1]+y_c,linewidth=2.5,linestyle='-',c='k')
+			
+			elif mode=='density':
+				# plot elliptical density
+				
+				for j,r in enumerate(np.linspace(0,sd_max_density,int(sd_max_density/dr))):
+					# plot ellipse
+					x = [r*sd[i][0]*np.cos(theta) for theta in np.arange(0,2*np.pi+h,h)]
+					y = [r*sd[i][1]*np.sin(theta) for theta in np.arange(0,2*np.pi+h,h)]
+					Z = np.transpose(axes[i]) @ np.array([x,y])
+
+					plt.gca().fill(Z[0]+x_c,Z[1]+y_c,c='b',alpha=alpha_max*np.exp(-np.sqrt(r)),edgecolor=None)
+			elif mode=='arrows':
+				# plot cluster center, a single elliptical contour, and arrows along the principal axes
+				pass
 		
 		
 	def generate_data(self,add_noise_vars=False, snr=None, p_over_n=None,seed=None,verbose=True):
@@ -221,10 +295,15 @@ class ClusterData:
 		self.cov_inv = cov_inv
 
 		if verbose: print('Place cluster centers...')
-		self.centers = self.center_geom.place_centers(self,seed=seed,verbose=verbose)
+		self.centers = (self.center_geom).place_centers(self,seed=seed, verbose=False)
+
+		# compute and report the max and min overlaps beween clusters
+		overlap_mat = CenterGeom.overlap_matrix(self.centers,self.cov_inv)
+		self.alpha_min_obs = CenterGeom.alpha_min_obs(overlap_mat)
+		self.alpha_max_obs = CenterGeom.alpha_max_obs(overlap_mat)
 
 		if verbose: print('Sample data...')
-		self.data, self.labels = self.data_dist.sample(self,seed=seed)
+		self.data, self.labels = (self.data_dist).sample(self,seed=seed)
 
 		if verbose: print('Success!')
 
@@ -234,7 +313,7 @@ class ClusterData:
 		return (self.data, self.labels)
 
 
-	def add_noise(self, data, snr, p_over_n, model='unif', margin=0.1, 
+	def add_noise(self, data, snr, p_over_n, model='gauss_mimic', margin=0.1, 
 		print_warning=True,seed=None):
 		"""
 		Add noise features to given data.
@@ -274,10 +353,6 @@ class ClusterData:
 		n_dim = data.shape[1] 
 		n_samples = data.shape[0]
 
-		# get lower and upper limits for each observation across features
-		low = np.min(data,axis=1)[:,np.newaxis]
-		high = np.max(data,axis=1)[:,np.newaxis]
-
 		if (not snr) and (not p_over_n):
 			raise Exception('Must provide either snr or p_over_n to determine ' +
 							'number of noise features to add.')
@@ -285,28 +360,36 @@ class ClusterData:
 			if snr and not p_over_n:
 				# add enough noise features to meet snr threshold
 				n_noise_vars = int(np.ceil(n_dim / snr))
-				noise_vars = np.random.uniform((1-margin)*low,(1+margin)*high,
-											  size=(n_samples,n_noise_vars))
+
 			if p_over_n and not snr:
 				# add enough noise features to meet p_over_n threshold
 				if n_dim/n_samples <= p_over_n:
 					n_noise_vars = int(np.ceil(p_over_n * n_samples)) - n_dim
-					noise_vars = np.random.uniform((1-margin)*low,(1+margin)*high,
-											  		 size=(n_samples,n_noise_vars))
 				else:
 					noise_vars = []
 					if print_warning:
 						print('Warning: data already sufficiently high-dimensional,' +
-						  	  ' no noise features added.')
+							  ' no noise features added.')
 			if snr and p_over_n:
 				# add enough noise features to meet both snr and p_over_n thresholds
 				n_noise_vars = np.max([int(np.ceil(n_dim/snr)), 
 									   int(np.ceil(n_samples*p_over_n))-n_dim])
-				noise_vars = np.random.uniform((1-margin)*low,(1+margin)*high,
-											   size=(n_samples,n_noise_vars))
+			
+			# create noise
+			feature_std = np.std(data,axis=0)
+			feature_mean = np.mean(data,axis=0)
+
+			# take a random feature, construct a Gaussian noise feature with the same mean+std
+			Z = np.zeros((n,noise_dim))
+
+			for i in range(noise_dim):
+				feature_idx = np.random.randint(0,p)
+				mean = feature_mean[feature_idx]
+				std = feature_std[feature_idx]
+				Z[:,i] = np.random.normal(loc=mean,scale=std,size=n)
 
 			# return data with noise features added
-			return np.concatenate([data,noise_vars],axis=1)
+			return np.concatenate([data,Z],axis=1)
 
 
 class CovGeom:
@@ -375,6 +458,8 @@ class CenterGeom:
 	-------
 	__init__(self) : abstract method
 	make_centers(self, clusterdata) : abstract method
+	compute_overlap : 
+	overlap_matrix : 
 
 	"""
 
@@ -384,6 +469,7 @@ class CenterGeom:
 								  ' e.g. centers.BoundedSepCenters, or code your'+
 								  ' own algorithm for placing cluster centers.')
 
+
 	def make_centers(self, clusterdata):
 		raise NotImplementedError('Cannot sample cluster centers from abstract'+
 								  ' CenterGeom. Choose a provided implementation,'+
@@ -391,6 +477,59 @@ class CenterGeom:
 								  ' own algorithm for placing cluster centers.')
 
 
+	def compute_overlap(center_1,cov_inv_1,center_2,cov_inv_2):
+		"""
+		Compute the level alpha of overlap between two clusters.
+		"""    
+		# compute the mahalanobis distances and calculate the chi2_quantile
+		dist1 = mahalanobis(center_1,center_2,cov_inv_1)
+		dist2 = mahalanobis(center_1,center_2,cov_inv_2)
+		chi2_quantile = (1/((1/dist1) + (1/dist2)))**2
+
+		# compute probability P(chi2 <= chi2_quantile) and get alpha
+		df = len(center_1)
+		alpha = 1 - (stats.chi2(df=df).cdf(chi2_quantile))
+
+		return alpha
+	
+
+	def overlap_matrix(centers,cov_inv):
+		"""
+		Compute the matrix of overlaps between pairs of clusters.
+		"""
+		n_centers = centers.shape[0]
+		pw_overlap = np.zeros((n_centers,n_centers))
+
+		for i in range(n_centers):
+			for j in range(i+1,n_centers):
+				pw_overlap[i,j] = CenterGeom.compute_overlap(centers[i,:], cov_inv[i],
+																	 centers[j,:], cov_inv[j])
+				#print('overlap', pw_overlap[i,j], ' by ', i,j)
+
+		pw_overlap = pw_overlap + np.transpose(pw_overlap)
+
+		return pw_overlap
+
+	def alpha_max_obs(overlap_mat):
+		"""
+		Compute maximum overlap between pairs of cluster centers.
+		"""
+		return np.max(overlap_mat.flatten())
+
+	def alpha_min_obs(overlap_mat):
+		"""
+		Compute the minimax overlap between cluster centers. That is,
+		for each cluster center, compute the maximum overlap (overlap with
+		the nearest other center) and take the minimum of these overlaps.
+		The resulting metric quantifies the degree of isolation of the most
+		isolated cluster.
+		"""
+		n_centers = overlap_mat.shape[0]
+		Z = overlap_mat.copy()
+		#Z[np.arange(n_centers),np.arange(n_centers)] = np.Inf
+
+		return np.min(np.max(Z,axis=1))
+
 
 class ClassBal:
 	"""
@@ -461,7 +600,7 @@ class DataDist:
 			# Sample data
 			X[start:end,:] = self.sample_cluster(class_size=class_sizes[i], 
 										  mean=centers[i], axes=axes[i],
-										  sd=sd[i],seed=seed+i)
+										  sd=sd[i],seed=seed+i if seed else seed)
 			start = end
 
 		return (X, y)
