@@ -86,7 +86,7 @@ class MaxMinClusters(ClusterData):
 	"""
 
 	def __init__(self, n_clusters=6, n_dim=2, n_samples=500, imbal_maxmin=2,
-				 aspect_maxmin=2, radius_maxmin=3, min_sep=0.95, max_sep=0.999, 
+				 aspect_maxmin=2, radius_maxmin=3, alpha_max=0.05, alpha_min=1e-3, 
 				 aspect_ref=1.5, packing=0.1,dist='gaussian',df=1, scale=1.0):
 		"""
 		Create a MaxMinClusters object. 
@@ -145,7 +145,7 @@ class MaxMinClusters(ClusterData):
 
 		cov_geom = MaxMinCov(ref_aspect=aspect_ref, aspect_maxmin=aspect_maxmin, 
 							 radius_maxmin=radius_maxmin)
-		center_geom = BoundedSepCenters(min_sep=min_sep,max_sep=max_sep, 
+		center_geom = BoundedSepCenters(alpha_max=alpha_max,alpha_min=alpha_min, 
 										packing=packing)
 		class_bal = MaxMinBal(imbal_ratio=imbal_maxmin)
 
@@ -263,10 +263,17 @@ class MaxMinCov(CovGeom):
 		out : ndarray
 			Cluster radii
 		"""
-		min_radius = (2*(ref_radius**n_dim)/(1 + self.radius_maxmin**n_dim))**(1/n_dim)
-		f = lambda r: (2*(ref_radius**n_dim) - (r**n_dim))**(1/n_dim)
-		return maxmin_sampler(n_clusters, ref_radius, min_radius, self.radius_maxmin, f,
-							  seed=seed)
+		log_min_radius = np.log(ref_radius) - (1/2)*np.log(self.radius_maxmin)
+		f = lambda log_r: 2*np.log(ref_radius) - log_r
+
+		#min_radius = (2*(ref_radius**n_dim)/(1 + self.radius_maxmin**n_dim))**(1/n_dim)
+		#f = lambda r: (2*(ref_radius**n_dim) - (r**n_dim))**(1/n_dim)
+
+		log_max_radius = np.log(ref_radius) + (1/2)*np.log(self.radius_maxmin)
+		maxmin_log_ratio = log_max_radius/log_min_radius
+
+		return np.exp(maxmin_sampler(n_clusters, np.log(ref_radius), log_min_radius, maxmin_log_ratio, f,
+							  seed=seed))
 	
 
 	def make_axis_sd(self, n_axes, sd, aspect, seed=None):
@@ -326,7 +333,8 @@ class MaxMinCov(CovGeom):
 			Matching indices of the output lists refer to the same cluster.
 
 		"""
-		np.random.seed(seed)
+		if seed:
+			np.random.seed(seed)
 
 		axis = list()
 		sd = list()
@@ -337,14 +345,14 @@ class MaxMinCov(CovGeom):
 		n_dim = clusterdata.n_dim
 		scale = clusterdata.scale
 		
-		cluster_radii = self.make_cluster_radii(n_clusters, scale, n_dim, seed=seed+2*n_clusters)
-		cluster_aspects = self.make_cluster_aspects(n_clusters,seed=seed+3*n_clusters)
+		cluster_radii = self.make_cluster_radii(n_clusters, scale, n_dim, seed=seed+2*n_clusters if seed else seed)
+		cluster_aspects = self.make_cluster_aspects(n_clusters,seed=seed+3*n_clusters if seed else seed)
 		
 		for clust in range(n_clusters):
 			# compute principal axes for cluster
-			axes = self.make_orthonormal_axes(n_dim, n_dim, seed=seed+clust)
+			axes = self.make_orthonormal_axes(n_dim, n_dim, seed=seed+clust if seed else seed)
 			axis_sd = self.make_axis_sd(n_dim, cluster_radii[clust], cluster_aspects[clust],
-										seed=seed+clust)
+										seed=seed+clust if seed else seed)
 
 			axis.append(axes)
 			sd.append(axis_sd)
